use constant TESTS => 9;
use Test::More tests => TESTS;

use strict;
use File::Temp qw(tempfile);
use Config::Extend::MySQL;

use DE_EPAGES::Core::API::File qw( ExistsFile DeleteFile CopyFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Database::API::BackupMySQL qw( MySQLShowVariables MyCnfDefault
                                              SetMySQLConf GetMySQLConf );

sub Main {
    my ($dbi) = @_;

    my (undef, $TmpFile) = tempfile();

    my $hSettings = MySQLShowVariables(GetTestSite());
    my $MyCnf = MyCnfDefault($hSettings->{basedir});
    ok( ExistsFile($MyCnf), "MyCnfDefault: config file exists ($MyCnf)" );

    CopyFile($MyCnf, $TmpFile);
    ok( GetMySQLConf($TmpFile)->{foobar} eq '',
        "GetMySQLConf: initially foobar is not defined in $TmpFile" );

    SetMySQLConf('foobaz','foobar', 1, {my_cnf => $TmpFile});
    # sections other than server/mysqld/mysqld_safe are ignored by GetMySQLConf()
    ok( GetMySQLConf($TmpFile)->{foobar} eq undef,
        "GetMySQLConf: [foobaz]foobar not found with GetMySQLConf() in $TmpFile" );

    # but the value can be read by Config::Extend::MySQL
    my $IniConfig = Config::Extend::MySQL->new({ from => $TmpFile,
                                                 using => 'Config::IniFiles' });
    ok( $IniConfig->val('foobaz','foobar') eq '1',
        "GetMySQLConf: [foobaz]foobar found with Config::Extend::MySQL in $TmpFile" );

    SetMySQLConf('mysqld','foobar', 0, {my_cnf => $TmpFile});
    # stangely enough, if sth. is set to 0 then GetMySQLConf() returns undef
    ok( GetMySQLConf($TmpFile)->{foobar} eq undef,
        "GetMySQLConf: foobar is set to '0' in $TmpFile" );

    SetMySQLConf('mysqld','foobar', 'yes', {my_cnf => $TmpFile});
    # stangely enough, if sth. is set to yes then GetMySQLConf() returns undef
    ok( GetMySQLConf($TmpFile)->{foobar} eq undef,
        "GetMySQLConf: foobar is set to 'yes' in $TmpFile" );

    SetMySQLConf('mysqld','foobar', 1, {my_cnf => $TmpFile});
    ok( GetMySQLConf($TmpFile)->{foobar} eq '1',
        "GetMySQLConf: foobar is set to '1' in $TmpFile" );

    DeleteFile($TmpFile);
}

RunScript(
    Debug => 1,
    Sub => sub {
        RunOnStore( Store => GetTestSite(), Sub => sub {
            my $dbi = GetCurrentDBHandle();
            SKIP: {
                # skip 1 test if not Sybase
                skip( "No MySQL DBD driver\n", TESTS ) unless $dbi->driverName eq 'mysql';

                ok( Init( $dbi ), 'Init' );
                Main( $dbi );
                ok( Cleanup( $dbi ), 'Cleanup' );
            }
        });
    }
);

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do( 'CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL, dt DATETIME NULL )' );

    return $dbi->existsTableColumn( 'dbitest', 'name' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->do( 'DROP TABLE dbitest' ) if $dbi->existsTable( 'dbitest' );
    return 1;
}

1;
