#========================================================================================
# package      DE_EPAGES::EasyMarketing::t::unit::ConsumerCreator
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::EasyMarketing::API::ConsumerCreator
#========================================================================================
package DE_EPAGES::EasyMarketing::t::unit::ConsumerCreator;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::EasyMarketing::API::ConsumerCreator;
use DE_EPAGES::Core::API::Error qw ( Error );
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_ConsumerCreator_NoConsumerYetCreated();
    $self->test_ConsumerCreator_GetConsumerRequestFailsWithNon404();
    $self->test_ConsumerCreator_ConsumerAlreadyCreated();

    done_testing();
}



sub test_ConsumerCreator_NoConsumerYetCreated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ConsumerKey = 'TestKey';
    my $ConsumerSecret = 'TestSecret';
    my $ConsumerClient = $self->_createConsumerClient_getConsumerRequestFails('404 Not Found');
    my $ConsumerCreator = $self->_createConsumerCreator($ConsumerClient);

    lives_ok(sub {
        $ConsumerCreator->createConsumerInEpagesJ($ConsumerKey, $ConsumerSecret);
    }, "$SubTestName - test if createConlsumerInEpagesJ was executed without exceptions");

    my $aLogWarningParameters = GetParametersFromMockifyCall($ConsumerCreator, '_logWarning');
    my $WarnMessage = $aLogWarningParameters->[0];
    is($WarnMessage, 'Create new consumer TestKey in ePagesJ.', "$SubTestName - test if log message is as expected");

    my $aSaveConsumerParameters = GetParametersFromMockifyCall($ConsumerClient, 'saveConsumer');
    my $Consumer = $aSaveConsumerParameters->[0];
    is($Consumer->key(), $ConsumerKey, "$SubTestName - test if consumer will be created with right key.");
    is($Consumer->secret(), $ConsumerSecret, "$SubTestName - test if consumer will be created with right secret.");
    is($Consumer->name(), 'EasyMarketing', "$SubTestName - test if consumer will be created with right name.");
}



sub test_ConsumerCreator_GetConsumerRequestFailsWithNon404 {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ConsumerKey = 'TestKey';
    my $ConsumerSecret = 'TestSecret';
    my $ConsumerClient = $self->_createConsumerClient_getConsumerRequestFails('666 Error');
    my $ConsumerCreator = $self->_createConsumerCreator($ConsumerClient);

    throws_ok(sub {
        $ConsumerCreator->createConsumerInEpagesJ($ConsumerKey, $ConsumerSecret);
    }, qr/REQUEST_FAILED: REQUEST_FAILED: StatusLine=666/,
    "$SubTestName - test if createConsumerInEpagesJ forwards errors which are not 404-Not-Found.");

    is(WasCalled($ConsumerCreator, '_saveConsumerInEpagesJ'), 0, "$SubTestName - test if _saveConsumerInEpagesJ was not called.");
}



sub test_ConsumerCreator_ConsumerAlreadyCreated {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ConsumerKey = 'TestKey';
    my $ConsumerSecret = 'TestSecret';
    my $ConsumerClient = $self->_createConsumerClient();
    my $ConsumerCreator = $self->_createConsumerCreator($ConsumerClient);

    lives_ok(sub {
        $ConsumerCreator->createConsumerInEpagesJ($ConsumerKey, $ConsumerSecret);
    }, "$SubTestName - test if createConsumerInEpagesJ was executed without exceptions");

    my $aLogWarningParameters = GetParametersFromMockifyCall($ConsumerCreator, '_logWarning');
    my $WarnMessage = $aLogWarningParameters->[0];
    is($WarnMessage, 'Consumer TestKey already exists in ePagesJ!', "$SubTestName - test if log message is as expected");
}



#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------



sub _createConsumerCreator {
    my $self = shift;
    my ($ConsumerClient) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EasyMarketing::API::ConsumerCreator', []);

    $Mockify->addMockWithReturnValue('_createConsumerClient', $ConsumerClient);
    $Mockify->addMockWithReturnValueAndParameterCheck('_logWarning', undef, ['string']);
    $Mockify->addMethodSpy('_saveConsumerInEpagesJ');

    return $Mockify->getMockObject();
}



sub _createConsumer {
    my $self = shift;
    my $aParams = [
        'key'       => 'TestKey',
        'secret'    => 'TestSecret',
        'name'      => 'EasyMarketing'
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::OAuthTokenClient::API::Object::OAuthConsumer', $aParams);
    return $Mockify->getMockObject();
}



sub _createConsumerClient {
    my $self = shift;
    my ($StatusLine) = @_;
    my $aParams = [(
        'ConsumersUrl' => bless({}, undef),
        'ConsumerUriTemplate' => bless({}, undef),
    )];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient', $aParams);
    $Mockify->addMockWithReturnValueAndParameterCheck('getConsumer', $self->_createConsumer(), [{'string'=>'TestKey'}]);
    return $Mockify->getMockObject();
}



sub _createConsumerClient_getConsumerRequestFails {
    my $self = shift;
    my ($StatusLine) = @_;
    my $aParams = [(
        'ConsumersUrl' => bless({}, undef),
        'ConsumerUriTemplate' => bless({}, undef),
    )];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient', $aParams);
    $Mockify->addMock('getConsumer', sub {
        Error('REQUEST_FAILED', {
            'StatusLine' => $StatusLine
        });
    });
    $Mockify->addMockWithReturnValueAndParameterCheck('saveConsumer', undef, ['object']);
    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();